<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan"
                   exclude-result-prefixes="xalan" xmlns:nk="http://1060.org">
	<xsl:output method="xml" omit-xml-declaration="yes"/>
   <xsl:variable name="explorer">/tools/explore/</xsl:variable>
   <xsl:variable name="resolver">/tools/search/resolver?resolverID=searchDocResolver&amp;documentID=</xsl:variable>
   
   	<xsl:param name="hash" nk:class="java.lang.String"/>
   	<xsl:param name="shash" nk:class="java.lang.String"/>
	<xsl:param name="space" nk:class="java.lang.String"/>
	<xsl:param name="version" nk:class="java.lang.String"/>
	<xsl:param name="type" nk:class="java.lang.String"/>
	<xsl:param name="id" nk:class="java.lang.String"/>
	<xsl:param name="autogen" nk:class="java.lang.Boolean"/>
	<xsl:param name="bookEntry"/>
	<xsl:param name="javadoc"/>
	<xsl:variable name="isCore" select="$space='urn:org:netkernel:core:meta'"/>
	
	<xsl:template match="/meta">
		<div>
			
			<div class="row">
				<div class="col-sm-10"><xsl:call-template name="main"/></div>
				<div class="col-sm-2"><xsl:call-template name="sidepanel"/></div>
			</div>
			
		</div>	
	</xsl:template>

	<xsl:template name="main">
		<div class="well endpoint">
			<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Endpoint</div>
				</div>
			<table class="table-bordered" style="width: 100%; margin: 0">
				<tr>
					<td class="label">Name:</td><td ><xsl:value-of select="name"/></td>
				</tr><tr>
					<td  class="label">Description:</td><td><xsl:value-of select="desc"/></td>
				</tr><tr>
					<td  class="label">Id:</td><td><xsl:value-of select="$id"/>
						<xsl:if test="$autogen">
						(auto-generated - for more details see <a href="{$resolver}doc:logicalreference:module:standard:logical:endpoint#Standard_Endpoint_Metadata">here</a>)
						</xsl:if>
					</td>
				</tr><tr>
					<td  class="label">Category:</td><td><xsl:value-of select="$type"/></td>
				</tr>
			</table>
			</div>
			
			<xsl:choose>
				<xsl:when test="$type='overlay' or $type='transparent overlay'">
					<xsl:call-template name="overlay"/>
				</xsl:when>
				<xsl:when test="$type='transreptor' ">
					<xsl:call-template name="transreptor"/>
				</xsl:when>
				<xsl:when test="$type='accessor' ">
					<xsl:call-template name="accessor"/>
				</xsl:when>
				<xsl:when test="$type='accessor prototype' ">
					<xsl:call-template name="proto-accessor"/>
				</xsl:when>
				<xsl:when test="$type='transport' ">
					<xsl:call-template name="transport"/>
				</xsl:when>
				<xsl:when test="$type='representation' ">
					<xsl:call-template name="representation"/>
				</xsl:when>
			</xsl:choose>
				
			<xsl:call-template name="import">
				<xsl:with-param name="type" select="$type"/>
			</xsl:call-template>
		</div>
	</xsl:template>
	
	<xsl:template name="sidepanel">
		<div class="callout-explorer">
			<div class="template-lbi">
				<a href="/tools/explore/{$hash}">
					<div><img border="0" src="/tools/introspect/pub/SpaceExplorer.png"/></div>
					<div>View in Space Explorer</div>
				</a>
			</div>
		</div>
		
		<xsl:if test="$bookEntry">
			<div class="callout-book"><div class="template-lbi">
				<a href="/book/view/{$bookEntry/book/id}/">
					<div><img src="/book/icon/{$bookEntry/book/id}" width="48" height="48" /></div>
					<div>View <xsl:value-of select="$bookEntry/book/title"/> Book</div>
				</a>
			</div></div>	
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template name="import">
		<xsl:param name="type"/>
		<xsl:if test="string-length($space) &gt; 0">
			<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Import Requirements</div>
				</div>
			<xsl:choose>
				<xsl:when test="$space!='urn:org:netkernel:core:meta'">
					<p>To use <xsl:value-of select="name"/><xsl:text> </xsl:text><xsl:value-of select="$type"/> you must import the module
					<a href="/tools/explore/{$shash}"><b><xsl:value-of select="$space"/></b></a>:
					{xml}
					<import>
						<uri><xsl:value-of select="$space"/></uri>
					</import>
					{/xml}
					</p>
				</xsl:when>
				<xsl:otherwise>
					<p>No import is necessary as this <xsl:value-of select="$type"/> is part of the core
					libraries provided by the kernel, layer0 and standard module.
					</p>
				</xsl:otherwise>
			</xsl:choose>
			</div>
			
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="overlay">
		<p><b><xsl:value-of select="name"/></b> is
			<xsl:choose>
				<xsl:when test="$type='transparent overlay'"> a transparent </xsl:when>
				<xsl:otherwise> an </xsl:otherwise>
			</xsl:choose>
		<a href="{$resolver}doc:glossary:overlay">overlay</a>.
		You must instantiate an instance of the overlay from its prototype, this will create a new instance within your application space.
		</p>

		<xsl:call-template name="parameters"/>
		<xsl:call-template name="proto-instantiate-example"/>

	</xsl:template>
	
	<xsl:template name="transreptor">
		<p>The <xsl:value-of select="name"/> is a <a href="{$resolver}doc:glossary:transreptor">transreptor</a>. The following table shows the
		transreption permutations that it will perform:</p>
		<table class="table-bordered" style="background: white;">
			<tr>
				<th>From (any of)</th><th>To (any of)</th>
			</tr>

			<xsl:call-template name="transrept-row">
				<xsl:with-param name="row" select="1"/>
			</xsl:call-template>
		</table>		
	</xsl:template>
	
	<xsl:template name="transrept-row">
		<xsl:param name="row"/>
		<xsl:variable name="from" select="(interface/args/arg[name='primary']/reps/rep)[$row]"/>
		<xsl:variable name="to" select="(interface/reps/rep)[$row]"/>
		<tr>
			<td>
				<xsl:for-each select="$from">
					<xsl:call-template name="rep"/>
				</xsl:for-each>
			</td>
			<td>
				<xsl:for-each select="$to">
					<xsl:call-template name="rep"/>
				</xsl:for-each>
			</td>
		</tr>
		<xsl:if test="$from or $to">
			<xsl:call-template name="transrept-row">
				<xsl:with-param name="row" select="$row+1"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="rep">
		<xsl:choose>
			<xsl:when test="name"><a href="{$explorer}{hash}"><xsl:value-of select="name"/></a></xsl:when>
			<xsl:otherwise><xsl:value-of select="class"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="accessor">
		<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Identifier Syntax</div>
				</div>
		<xsl:variable name="isActive" select="interface/grammar/group[group[1]/@name='scheme' and *[2]/@name='activeType']"/>
		<xsl:choose>
			<xsl:when test="$isActive">
				<xsl:call-template name="activeURI"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="regularGrammar"/>
			</xsl:otherwise>
		</xsl:choose>
		</div>
		
		<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Request Verbs</div>
				</div>
		<p>The following	
			<xsl:choose>
				<xsl:when test="count(interface/verbs/verb) > 1"> verbs are </xsl:when>
				<xsl:otherwise> verb is </xsl:otherwise>
			</xsl:choose>supported:
		</p>
		
		<table class="table-bordered">
			<tr>
				<td colspan="8">Verb<xsl:if test="count(interface/verbs/verb) > 1"><xsl:text>s</xsl:text></xsl:if></td>
			</tr>
			<tr>
			<xsl:for-each select="interface/verbs/verb">
				<td>
					<a href="#{text()}"><xsl:value-of select="."/></a>
				</td>
			</xsl:for-each>
			</tr>
		</table>
		</div>
		
		<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Response</div>
				</div>
		
		<xsl:choose>
			<xsl:when test="interface/reps/rep">
				<p>The following response representations can be returned for SOURCE requests:</p>
				<table class="table-bordered">
					<tr>
						<th colspan="8">SOURCE Representations</th>
					</tr>
					<tr>
						<xsl:for-each select="interface/reps/rep">
							<td><xsl:call-template name="rep"/>
							</td>
						</xsl:for-each>
					</tr>
				</table>
			</xsl:when>
			<xsl:otherwise><p>The response representation of this accessor for SOURCE requests is unknown.</p></xsl:otherwise>
		</xsl:choose>
		
		<xsl:choose>
			<xsl:when test="interface/exs/ex">
				<p>The following exceptions can be thrown from this accessor:</p>
				<table class="table-bordered">
					<tr>
						<th>Exception Id</th><th>Description</th>
					</tr>
					<xsl:for-each select="interface/exs/ex">
						<tr>
							<td><xsl:value-of select="name"/></td>
							<td><xsl:value-of select="desc"/></td>
						</tr>
					</xsl:for-each>
				</table>
			</xsl:when>
			<xsl:otherwise><p>This accessor throws no documented exceptions.</p></xsl:otherwise>
		</xsl:choose>
		</div>
		
	</xsl:template>
	
	<xsl:template name="regularGrammar">
		<p><b><xsl:value-of select="name"/></b> is an <a href="{$resolver}doc:glossary:accessor">accessor</a> using a
		<a href="{$resolver}doc:logicalreference:module:standard:logical:grammar#Standard_Identifier_Grammars">standard identifier grammar</a>:</p>
		<div id="grammar" >
		{xml}
		<xsl:copy-of select="//grammar"/>
		{/xml}
		</div>
		<p>with the following arguments: (for more details on argument passing details see  <a href="{$resolver}doc:logicalreference:module:standard:logical:identifier:arguments">here</a>)</p>
		<table class="table-bordered">
			<tr>
				<th>Argument</th>
				<th>Rules</th>
				<th>Typing</th>
				<th>Description</th>
			</tr>
			<xsl:for-each select="interface/grammar//*[@name]">
				<xsl:variable name="name" select="@name"/>
				<xsl:variable name="argMeta" select="/meta/interface/args/arg[name=$name]"/>
				<tr>
					<td><b><xsl:value-of select="@name"/></b></td>
					<td>
						<xsl:if test="$argMeta/default">
							<div>Defaults to &quot;<xsl:value-of select="$argMeta/default"/>&quot;</div>
						</xsl:if>
					</td>
					<td>
						<xsl:for-each select="$argMeta"><xsl:call-template name="arg-typing"/></xsl:for-each>
					</td>
					<td>
						<xsl:choose>
							<xsl:when test="$argMeta/desc"><xsl:value-of select="$argMeta/desc"/></xsl:when>
							<xsl:otherwise><i>(no description available)</i></xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
			</xsl:for-each>
			<xsl:if test="not(interface/grammar//*[@name])">
				<tr><td colspan="4">No arguments defined</td></tr>
			</xsl:if>
		</table>
			
		
	</xsl:template>

	<xsl:template name="activeURI">
		<p><b><xsl:value-of select="name"/></b> is an accessor using <a href="{$resolver}doc:glossary:active">Active URI</a> syntax with the following base identifiers:
		</p>
		<xsl:variable name="scheme" select="interface/grammar/group/group[1]"/>
		<table class="table-bordered">
			<tr>
				<th>Base</th>
			</tr>
			<xsl:for-each select="interface/grammar/group/choice/group | interface/grammar/group/group[2]">
				<tr>
					<td><xsl:value-of select="$scheme"/>:<xsl:value-of select="."/></td>
				</tr>
			</xsl:for-each>
		</table>
		<xsl:choose>
			<xsl:when test="interface/grammar/group/interleave">
		<p>and the following arguments: (for more details on argument passing details see  <a href="{$resolver}doc:logicalreference:module:standard:logical:identifier:arguments">here</a>)</p>
		<table class="table-bordered">
			<tr>
				<th>Argument</th>
				<th>Rules</th>
				<th>Typing</th>
				<th>Description</th>
			</tr>
			<xsl:for-each select="interface/grammar/group/interleave/group[not(last() and count(group)=2)]">
				<tr>
					<xsl:variable name="name" select="group/@name"/>
					<xsl:variable name="argMeta" select="/meta/interface/args/arg[name=$name]"/>
					<td><b><xsl:value-of select="$name"/></b></td>
					<td>	<div>
							<xsl:choose>
								<xsl:when test="(@min='1' or not(@min)) and (@max='1' or not(@max))">Mandatory</xsl:when>
								<xsl:when test="@min='0' and (@max='1' or not(@max))">Optional</xsl:when>
								<xsl:when test="@min='0' and @max&gt;'32768'">Any quantity</xsl:when>
								<xsl:otherwise><xsl:choose><xsl:when test="@min"><xsl:value-of select="@min"/></xsl:when><xsl:otherwise>1</xsl:otherwise></xsl:choose> ...
									<xsl:choose>
										<xsl:when test="@max='*'">infinity</xsl:when>
										<xsl:otherwise><xsl:value-of select="@max"/></xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
							</div>
							<xsl:if test="$argMeta/default">
								<div>Defaults to &quot;<xsl:value-of select="$argMeta/default"/>&quot;</div>
							</xsl:if>
					</td>
					<td>
						<xsl:for-each select="$argMeta"><xsl:call-template name="arg-typing"/></xsl:for-each>
					</td>
					<td>
						<xsl:choose>
							<xsl:when test="$argMeta/desc"><xsl:value-of select="$argMeta/desc"/></xsl:when>
							<xsl:otherwise><i>(no description available)</i></xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
			</xsl:for-each>
			<xsl:if test="interface/grammar/group/interleave/group[last() and count(group)=2]">
				<xsl:variable name="argMeta" select="/meta/interface/args/arg[name='varargs']"/>
				<tr>
					<td><b>varargs
					<xsl:if test="$argMeta/default"> [<xsl:value-of select="$argMeta/default"/>]
					</xsl:if>
					</b></td>
					<td>Any quantity</td>
					<td>Representation (java.lang.Object)</td>
					<td>
						<xsl:choose>
							<xsl:when test="$argMeta/desc"><xsl:value-of select="$argMeta/desc"/></xsl:when>
							<xsl:otherwise>
								<a href="{$resolver}doc:logicalreference:module:standard:logical:grammar#varargs">Varargs</a> allows an arbitrary number of additional unspecified arguments.
							</xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
			</xsl:if>
		</table>
		</xsl:when>
		<xsl:otherwise>
			<p>and no arguments can be specified on the identifier.</p>
		</xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>

	<xsl:template name="arg-typing">
		<xsl:choose>
			<xsl:when test="type='STRING'">String (<xsl:value-of select="substringType"/>)</xsl:when>
			<xsl:when test="type='REPRESENTATION'">Representation (<xsl:for-each select="reps/rep">
					<xsl:call-template name="rep"/>
					<xsl:if test="position()!=last()">, </xsl:if>
				</xsl:for-each>)</xsl:when>
			<xsl:otherwise>Identifier</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<xsl:template name="proto-accessor">
		<!--<h3>Prototype Accessors</h3>-->
		<p><b><xsl:value-of select="name"/></b> is an <a href="{$resolver}doc:logicalreference:module:standard:logical:accessor">accessor</a> that must be instantiated
		from a <a href="{$resolver}doc:logicalreference:module:standard:logical:prototype">prototype</a>.
		You must instantiate the prototype in your module to create an accessor that can receive requests.
		</p>
		<xsl:call-template name="parameters"/>
		<xsl:call-template name="proto-instantiate-example"/>		
	</xsl:template>
	
	<xsl:template name="transport">
		<p><b><xsl:value-of select="name"/></b> is a <a href="{$resolver}doc:logicalreference:module:standard:logical:transport">transport</a>
		that must be instantiated from a <a href="{$resolver}doc:logicalreference:module:standard:logical:prototype">prototype</a>.</p>
		
		<xsl:call-template name="parameters"/>
		<xsl:call-template name="proto-instantiate-example"/>
		
	</xsl:template>	
	
	<xsl:template name="representation">
		<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Interface</div>
				</div>
		<p><xsl:value-of select="name"/> is a <a href="{$resolver}doc:glossary:representation">representation</a>
		implemented by the Java class/interface <b><xsl:value-of select="implementation"/></b>.
		</p>
		<xsl:choose>
		<xsl:when test="$javadoc">
			<div class="javadoc"><xsl:copy-of select="$javadoc"/></div>
		</xsl:when>
		<xsl:otherwise>
			<div class="javadoc">
				<p><img src="/tools/javadoc/img/icon.png" align="middle"/>
					Javadoc may be available but not built for this representation. <a href="/tools/javadoc/">Build here</a>.</p>
			</div>
		</xsl:otherwise>
		</xsl:choose>
		</div>
	</xsl:template>
	
	<xsl:template name="proto-instantiate-example">
		<xsl:if test="$space!='urn:org:netkernel:core:meta'">
		<p>Here is an auto-generated example of how to instantiate an instance of <xsl:value-of select="name"/>:</p>
		{xml}

		<xsl:variable name="ename">
			<xsl:choose>
				<xsl:when test="$type='overlay' or $type='transparent overlay'">overlay</xsl:when>
				<xsl:when test="$type='transreptor' ">transreptor</xsl:when>
				<xsl:when test="$type='accessor' or  $type='accessor prototype'">accessor</xsl:when>
				<xsl:when test="$type='transport' ">transport</xsl:when>
				<xsl:otherwise>endpoint</xsl:otherwise>
			</xsl:choose>	
		</xsl:variable>

		<xsl:element name="{$ename}">
			<prototype><xsl:value-of select="identifier"/></prototype>
			<xsl:for-each select="parameters/parameter[name!='space' and name!='private' and name!='id' and min&gt;0 ]">
				<xsl:element name="{name}">
					<xsl:choose>
						<xsl:when test="desc"><xsl:value-of select="desc"/></xsl:when>
						<xsl:when test="name='private'"/>
						<xsl:when test="name='id'"/>
						<xsl:when test="name='grammar'">an identifier grammar definition</xsl:when>
						<xsl:when test="contains(class,'ISpace')">a nested space</xsl:when>
						<xsl:when test="contains(class,'IIdentifier')">a resource identifier or literal xml fragment</xsl:when>
						<xsl:when test="contains(class,'Integer')">an integer value</xsl:when>
						<xsl:when test="contains(class,'Long')">a long value</xsl:when>
						<xsl:when test="contains(class,'String')">a string value</xsl:when>
						<xsl:when test="contains(class,'Float')">a float value</xsl:when>
						<xsl:when test="contains(class,'Double')">a double value</xsl:when>
						<xsl:when test="contains(class,'Boolean')">a boolean value</xsl:when>
						<xsl:when test="contains(class,'Element')">a literal xml fragment</xsl:when>
					</xsl:choose>
				</xsl:element>
			</xsl:for-each>
			<xsl:if test="parameters/parameter[name='space']">
				<space>
					<xsl:comment>wrapped space...</xsl:comment>
				</space>
			</xsl:if>
		</xsl:element>
		{/xml}
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="parameters">
		<div class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Parameters</div>
				</div>
		<!--<p>Parameters are tags which customise the instantiation of the prototype.</p>-->
		<xsl:choose>
			<xsl:when test="parameters/parameter"> <!-- [not(contains(class,'ISpace'))] -->
			<p>The <b><xsl:value-of select="identifier"/></b> prototype has the following initialisation parameters:</p>
			<table class="table-bordered">
				<tr>
					<th>Name</th>
					<th>Rules</th>
					<th>Typing</th>
					<th>Default</th>
					<th>Description</th>
				</tr>
				<xsl:for-each select="parameters/parameter">
					<tr>
						<td ><b><xsl:value-of select="name"/></b></td>
						<td  >
							<xsl:choose>
								<xsl:when test="min='1' and max='1'">Mandatory</xsl:when>
								<xsl:when test="min='0' and max='1'">Optional</xsl:when>
								<xsl:when test="min='0' and max&gt;'32768'">Any quantity</xsl:when>
								<xsl:otherwise><xsl:value-of select="min"/> ...
									<xsl:choose>
										<xsl:when test="max&gt;'32768'">infinity</xsl:when>
										<xsl:otherwise><xsl:value-of select="max"/></xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</td>
						<td  >
						<xsl:choose>
								<xsl:when test="contains(class,'ISpace')">Space</xsl:when>
								<xsl:when test="contains(class,'IIdentifier')">Identifier or XML</xsl:when>
								<xsl:when test="contains(class,'Integer')">Integer</xsl:when>
								<xsl:when test="contains(class,'Long')">Long</xsl:when>
								<xsl:when test="contains(class,'String')">String</xsl:when>
								<xsl:when test="contains(class,'Float')">Float</xsl:when>
								<xsl:when test="contains(class,'Double')">Double</xsl:when>
								<xsl:when test="contains(class,'Boolean')">Boolean</xsl:when>
								<xsl:when test="contains(class,'Element')">XML</xsl:when>
							</xsl:choose>
						</td>
						<td>
							<xsl:choose>
								<xsl:when test="default='no default'"><i>(none)</i></xsl:when>
								<xsl:otherwise><xsl:value-of select="default"/></xsl:otherwise>
							</xsl:choose>
						
						</td>
						<td >
							<xsl:choose>
								<xsl:when test="desc"><div><xsl:value-of select="desc"/></div>
								</xsl:when>
								<xsl:when test="name='grammar'">
									<div>An <a href="{$resolver}doc:logicalreference:module:standard:logical:grammar">identifier grammar</a> definition.</div>
								</xsl:when>
								<xsl:when test="name='space'">
									<div>A nested space definition which the overlay will delegate all requests in to.</div>
								</xsl:when>
								<xsl:when test="name='id'">
									<div>An identifier for created logical endpoint, if omitted a unique auto-generated value it will be used.</div>
								</xsl:when>
								<xsl:when test="name='private'">
									<div>If included this parameter will mark the endpoint as private and it will not be exposed outside module.</div>
								</xsl:when>
								<xsl:otherwise><i>(no description available)</i>
								</xsl:otherwise>
							</xsl:choose>
							<div>
							</div>
						</td>
					</tr>
				</xsl:for-each>
			</table>
			</xsl:when>
			<xsl:otherwise><p>The <xsl:value-of select="name"/> has no initialisation parameters.</p></xsl:otherwise>
		</xsl:choose>
		</div>
	</xsl:template>
		
</xsl:stylesheet>
