package org.netkernel.doc.core.accessor;

import java.util.HashMap;
import java.util.Map;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class DocumentationAccessor extends StandardAccessorImpl
{
	public DocumentationAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	String type=context.getThisRequest().getArgumentValue("activeType");
		Object result=null;
		if(type.equals("documentationTree"))
		{	result=getTree(context);
		}
		else if(type.equals("documentationMap"))
		{	IHDSNode list=getTree(context);
			Map<String, IHDSNode> map=new HashMap<String, IHDSNode>(10);
			for(IHDSNode space : list.getNodes("spaces/space"))
			{	for(IHDSNode doc : space.getNodes("docs/doc"))
				{	HDSBuilder b=new HDSBuilder();
					b.pushNode("doc");
					b.addNode("space", space.getFirstValue("id"));
					b.addNode("version", space.getFirstValue("version"));
					b.importChildren(doc);
					b.popNode();
					map.put((String)doc.getFirstValue("id"), b.getRoot());
				}
			}
			result=map;
		}
		else if(type.equals("documentationEntry"))
		{	Map map=(Map)context.source("active:documentationMap");
			String id=context.getThisRequest().getArgumentValue("id");
			result=(IHDSNode)map.get(id);
			if (result==null)
			{	throw new NKFException("Documentation not Found","Document with id ["+id+"] not found - is module installed?");
			}
		}
		context.createResponseFrom(result);
	}

	private IHDSNode getTree(INKFRequestContext context) throws Exception
	{	INKFRequest req=context.createRequest("active:spaceAggregateHDS");
		req.addArgument("uri", "res:/etc/system/Docs.xml");
		req.setRepresentationClass(IHDSNode.class);
		return (IHDSNode)context.issueRequest(req);	
	}
	
}
