package org.netkernel.doc.endpoint;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.HDSUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class BookViewWrapper extends StandardAccessorImpl
{
	public BookViewWrapper()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{
		context.setCWU(null);
		String bookid=context.getThisRequest().getArgumentValue("bookid");
		INKFRequest req=context.createRequest("active:bookEntry");
		req.addArgument("id", bookid);
		req.setRepresentationClass(IHDSNode.class);
		IHDSNode book=(IHDSNode)context.issueRequest(req);
		if (book==null)
		{	throw new NKFException("book not installed",bookid);
		}
		String docid=context.getThisRequest().getArgumentValue("docid");
		if(docid.equals(""))
		{	//Default to the first document in the list
			docid=(String)book.getFirstValue("/book/toc/item[1]/@id");
		}

		IHDSNode rep;
		try
		{	req=context.createRequest("active:documentationEntry");
			req.addArgument("id", docid);
			rep=(IHDSNode)context.issueRequest(req);
		}
		catch (Exception e)
		{	//default to missing page rather than error
			req=context.createRequest("active:documentationEntry");
			req.addArgument("id", "doc:missing");
			rep=(IHDSNode)context.issueRequest(req);
			String title=null;
			for (IHDSNode b=book.getChildren()[0]; b!=null; b=HDSUtils.inOrderTraversalNext(b, book))
			{	if (b.getName().equals("@id") && docid.equals(b.getValue()))
				{	title=(String)b.getParent().getFirstValue("@title");
					break;
				}
			}
			if (title!=null)
			{	HDSBuilder b=new HDSBuilder();
				b.importChildren(rep);
				b.setCursor(b.getRoot().getFirstNode("/doc/title"));
				b.setValue(title);
				rep=b.getRoot();
			}
			
		}

		req=context.createRequest("active:docRenderer");
		req.addArgument("bookid", bookid);
		req.addArgument("docid", docid);
		req.addArgumentByValue("docentry", rep);
		INKFResponse resp=context.createResponseFrom(context.issueRequestForResponse(req));
		resp.setHeader("WrappedControlPanelNarrow", true);

	}
	
	
}
