package org.netkernel.doc.endpoint;

import java.net.URI;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.StandardSpace;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.impl.Version;

public class DocEditAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String docId=aContext.getThisRequest().getArgumentValue("docid");
		String bookId=aContext.getThisRequest().getArgumentValue("bookid");
		
		INKFRequest req=aContext.createRequest("active:documentationEntry");
		req.addArgument("id", docId);
		req.setRepresentationClass(IHDSNode.class);
		IHDSNode docentry=(IHDSNode)aContext.issueRequest(req);
		String src = (String)docentry.getFirstValue("doc/uri");
		String space=(String)docentry.getFirstValue("doc/space");
		String version=(String)docentry.getFirstValue("doc/version");
		Version v=new Version(version);
		ISpace s=getKernel().getSpace(new SimpleIdentifierImpl(space), v, v);		
		
		IHDSNode params=aContext.source("httpRequest:/params",IHDSNode.class);
		String text=(String)params.getFirstValue("text");
		if (text!=null)
		{
			URI modSrc=((StandardSpace)s).getOwningModule().getSource();
			int i=src.indexOf(":");
			String path=src.substring(i+2);
			String srcURI=modSrc.toString()+path;
			aContext.sink(srcURI, text);
			Thread.sleep(1000);
			
			String redirect="/book/view/"+bookId+"/"+docId;
			aContext.sink("httpResponse:/redirect",redirect);
		}
		else
		{
			req=aContext.createRequest(src);
			req.injectRequestScope(s);
			req.setRepresentationClass(String.class);
			String content=(String)aContext.issueRequest(req);
			
			content="<text>"+XMLUtils.escape(content)+"</text>";
			
			req=aContext.createRequest("active:xslt");
			req.addArgument("operator", "res:/org/netkernel/doc/endpoint/styleDocEdit.xsl");
			req.addArgumentByValue("operand", content);
			
			INKFResponseReadOnly respIn=aContext.issueRequestForResponse(req);
			INKFResponse resp=aContext.createResponseFrom(respIn);
			resp.setHeader("WrappedControlPanel",true);
			
		}
		
	}
}
