package org.netkernel.doc.map;

import java.util.*;

import org.netkernel.layer0.boot.BootPersistence;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class VisitDataAccessor extends StandardAccessorImpl
{
	private HashMap<String,VisitStruct> mDocToVisit = new HashMap<String,VisitStruct>();
	private String mVisualisation;
	
	public VisitDataAccessor()
	{	this.declareThreadSafe();
	}
	
	public void postCommission(INKFRequestContext aContext) throws Exception
	{
		IReadableBinaryStreamRepresentation rep=null;
		try
		{	rep=BootPersistence.sourceBootResource("docmap.xml", aContext.getKernelContext().getKernel());
		}
		catch (Exception e)
		{	// ignore - no saved state
		}
		if (rep!=null)
		{	IHDSNode n=aContext.transrept(rep, IHDSNode.class);
			for (IHDSNode e : n.getNodes("/docmap/e"))
			{	String id=(String)e.getFirstValue("id");
				int count=Integer.parseInt((String)e.getFirstValue("c"));
				long last=Long.parseLong((String)e.getFirstValue("l"));
				mDocToVisit.put(id, new VisitStruct(count,last));
			}
			mVisualisation=(String)n.getFirstValue("/docmap/visualisation");
		}
	}
	
	public void preDecommission(INKFRequestContext aContext) throws Exception
	{
		HDSBuilder b=new HDSBuilder();
		b.pushNode("docmap");
		b.addNode("visualisation",mVisualisation);
		for (Map.Entry<String, VisitStruct> entry : mDocToVisit.entrySet())
		{	b.pushNode("e");
			b.addNode("id", entry.getKey());
			b.addNode("c", Integer.toString(entry.getValue().getCount()));
			b.addNode("l", Long.toString(entry.getValue().getLast()));
			b.popNode();
		}
		IBinaryStreamRepresentation rep=aContext.transrept(b.getRoot(), IBinaryStreamRepresentation.class);
		BootPersistence.sinkBootResource("docmap.xml", rep, aContext.getKernelContext().getKernel());
	}
	
	public void onSink(INKFRequestContext aContext) throws Exception
	{	
		String type=aContext.getThisRequest().getArgumentValue(ParsedIdentifierImpl.ARG_ACTIVE_TYPE);
		 if (type.equals("docVisualisation"))
		{	onDocVisualisation(aContext);
		}
	}

	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String type=aContext.getThisRequest().getArgumentValue(ParsedIdentifierImpl.ARG_ACTIVE_TYPE);
		if (type.equals("docHit"))
		{	onDocHit(aContext);
		}
		else if (type.equals("docUsage"))
		{	onDocData(aContext);
		}
		else if (type.equals("docVisualisation"))
		{	onDocVisualisation(aContext);
		}
		else if (type.equals("docUsageReset"))
		{	onDocReset(aContext);
		}
			
	}
	
	private synchronized void onDocReset(INKFRequestContext aContext) throws Exception
	{	mDocToVisit.clear();
	}

	private synchronized void onDocVisualisation(INKFRequestContext aContext) throws Exception
	{
		int verb=aContext.getThisRequest().getVerb();
		if (verb==INKFRequestReadOnly.VERB_SINK)
		{	mVisualisation=aContext.sourcePrimary(String.class);
		}
		else if (verb==INKFRequestReadOnly.VERB_SOURCE)
		{	aContext.createResponseFrom(mVisualisation).setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
	}

	private synchronized void onDocHit(INKFRequestContext aContext) throws Exception
	{	String docId=aContext.getThisRequest().getArgumentValue("docId");
		//System.out.println("pageHIt: "+docId);
		VisitStruct vs=mDocToVisit.get(docId);
		if (vs==null)
		{	vs=new VisitStruct();
			mDocToVisit.put(docId, vs);
		}
		else
		{	vs.touch();
		}
	}
	
	private synchronized void onDocData(INKFRequestContext aContext) throws Exception
	{
		List<VisitStruct2> data=new ArrayList<VisitStruct2>(mDocToVisit.size());
		for (Map.Entry<String, VisitStruct> e: mDocToVisit.entrySet())
		{	VisitStruct2 s=new VisitStruct2(e.getKey(),e.getValue());
			data.add(s);
		}
		
		Comparator<VisitStruct2> c=new Comparator<VisitStruct2>()
		{	public int compare(VisitStruct2 o1, VisitStruct2 o2)
			{	return o2.mCount-o1.mCount;
			}
		};
		
		Collections.sort(data, c);
		
		Map<String,VisitStruct2> result=new HashMap<String, VisitStruct2>();
		int r=0;
		for (VisitStruct2 s : data)
		{	s.mRank=r++;
			result.put(s.mName,s);
		}
		
		INKFResponse response=aContext.createResponseFrom(result);
		response.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
}